#
# See the file "LICENSE" for the full license governing this code.
#
#
# This function determins the CIS product_version to use for this call. It uses the following algorithmn:
# 
# 1. Use te specified version for this database
# 2. Use a generic specified version for all databases
# 3. Check if fact pg_version is set. If not, return an undef
# 4. Use the version based on the version of the database
#
Puppet::Functions.create_function('mssql_secured::default_product_version') do
  dispatch :default_product_version do
    param 'String[1]', :db
  end

  def default_product_version(db)
    #
    # When a version is specified, use it.
    #
    value = lookup_setting('product_version', nil)
    return value if value
    #
    # When the version fact is available, use it
    #
    return nil unless closure_scope.include?('mssql_version')
    product_version = closure_scope['mssql_version']
    case product_version
    when /^15\..*/
      'mssql2019'
    when /^14\..*/
      'mssql2017'
    when /^13\..*/
      'mssql2016'
    when /^12\..*/
      'mssql2014'
    else
      fail "Database version #{product_version} not recognised"
    end
  end

  def lookup_setting(db, default_value)
    call_function('mssql_secured::lookup_setting', db, default_value )
  end


end
  