#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
require 'puppet'

$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../../mssql_secured/lib"))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../../easy_type/lib"))

require 'easy_type/preamble'
require 'easy_type/setup'
require 'puppet_x/enterprisemodules/mssql/access'
require 'puppet_x/enterprisemodules/mssql/sql'


module Puppet_X
  module EnterpriseModules
    # Doc
    module MsSql
      # Docs
      # TODO: Write documentation
      module Fact

        def mssql_array_fact_all(mappings = {}, &proc)
          query_routine = lambda { |query| sql_on_all_databases(query) }
          mapper        = lambda { |results, lambda_mappings| array_mapper(results, lambda_mappings) }
          mssql_fact_internal(query_routine, mapper, mappings, &proc)
        end

        def mssql_array_fact(mappings = {}, &proc)
          query_routine = lambda { |query| sql_on_all_master_databases(query) }
          mapper        = lambda { |results, lambda_mappings| array_mapper(results, lambda_mappings) }
          mssql_fact_internal(query_routine, mapper, mappings, &proc)
        end

        def mssql_hierarchical_fact(mappings = {}, &proc)
          query_routine = lambda { |query| sql_on_all_master_databases(query) }
          mapper        = lambda { |results, lambda_mappings| hierarchical_mapper(results, lambda_mappings) }
          mssql_fact_internal(query_routine, mapper, mappings, &proc)
        end

        def mssql_record_fact(mappings = {}, &proc)
          query_routine = lambda { |query| sql_on_all_master_databases(query) }
          mapper        = lambda { |results, lambda_mappings| record_mapper(results, lambda_mappings) }
          mssql_fact_internal(query_routine, mapper, mappings, &proc)
        end

        def mssql_fact_internal(query_routine, mapper, mappings = {})
          query = yield
          setcode do
            extend Puppet_X::EnterpriseModules::MsSql::Sql
            extend EasyType::Helpers
            results = query_routine.call(query)
            mapper.call(results, mappings)
          end
        end

        def hierarchical_mapper(results, mappings)
          instances = results.map {|e| e['instance']}.uniq
          returns = {}
          instances.each do | instance|
            returns[instance] = results.select {|e| e['instance'] == instance}.collect {|e| e[mappings[:key]]}
          end
          returns
        end
  
        def no_mappings(result)
          result.delete(:instance)
          result[result.keys.first]
        end


        def array_mapper(results, mappings)
          results.collect do |result|
            if mappings == {}
              no_mappings(result)
            else
              return_value = mappings.collect do |key, entry|
                [key.to_s, result[entry]]
              end.flatten
              Hash[*return_value]
            end
          end
        end

        def record_mapper(result, mappings)
          result = result.first
          if mappings == {}
            no_mappings(result)
          else
            return_value = mappings.collect do |key, entry|
              [key.to_s, result[entry]]
            end
            Hash[return_value]
          end
        end
      end
    end
  end
end
