#
# See the file "LICENSE" for the full license governing this code.
#
#
# Check all requirements for facts
#
require 'puppet_x/enterprisemodules/mssql/fact'
require 'puppet/feature/mssql_running'


def mssql_secured_fact(fact_name, type=:array)
  include Puppet_X::EnterpriseModules::MsSql::Fact
  include MssqlFeature
  if Puppet.features.mssql_running?
    begin
      require 'entitlement/entitlement'
      ::Entitlement::Entitlement.entitled?("mssql_secured#facts##{fact_name}")
      case type
      when :array
        Facter.add("mssql_#{fact_name}") do
          mssql_array_fact(value: 'value', database: 'database', instance: 'instance') do
            yield
          end
        end          
      when :hierarchical
        Facter.add("mssql_#{fact_name}") do
          mssql_hierarchical_fact(key: 'value') do
            yield
          end
        end
      when :record
        Facter.add("mssql_#{fact_name}") do
          mssql_record_fact(key: 'value') do
            yield
          end
        end
      else
        fail "Invalid type '#{type}'"
      end
    rescue
      Puppet.warn_once(:no_mssql_secured_entitlements, 'mssql_secured:Facts', 'mssql_secured not entitled on this node; Skipping mssql_secured facts')
      Facter.add("mssql_#{fact_name}") do
        setcode { [] } # return an empty value
      end
    end
  else
    Facter.add("mssql_#{fact_name}") do
      setcode { [] } # return an empty value
    end
  end
end
