#
# mssql_secured::controls::default_trace_enabled_server_configuration_option_is_set_to_1
#
# @summary The default trace provides audit logging of database activity including account creations, privilege elevation and execution of DBCC commands.
# Default trace provides valuable audit information regarding security-related activities on the server.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::default_trace_enabled_server_configuration_option_is_set_to_1: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::default_trace_enabled_server_configuration_option_is_set_to_1::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `default_trace_enabled_server_configuration_option_is_set_to_1` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 5.2
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 5.2
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 5.2
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 5.2
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::default_trace_enabled_server_configuration_option_is_set_to_1 {
  $instance = $title

  mssql_secured_setup { "default_trace_enabled_server_configuration_option_is_set_to_1 on ${title}":
    ensure => 'present',
  }
  -> sqlserver_tsql { "default_trace_enabled_server_configuration_option_is_set_to_1 on ${title}":
    instance => $instance,
    # lint:ignore:strict_indent
    command  => @("SQL"),
      EXECUTE sp_configure 'show advanced options', 1;
      RECONFIGURE;
      EXECUTE sp_configure 'default trace enabled', 1;
      RECONFIGURE;
      EXECUTE sp_configure 'show advanced options', 0;
      RECONFIGURE;
    SQL
    # lint:endignore
  }
}
