#
# mssql_secured::controls::must_change_option_is_set_to_on_for_all_sql_authenticated_logins
#
# @summary Whenever this option is set to ON, SQL Server will prompt for an updated password the first time the new or altered login is used.
# Enforcing a password change after a reset or new login creation will prevent the account administrators or anyone accessing the initial password from misuse of the SQL login created without being noticed.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::must_change_option_is_set_to_on_for_all_sql_authenticated_logins: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::must_change_option_is_set_to_on_for_all_sql_authenticated_logins::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `must_change_option_is_set_to_on_for_all_sql_authenticated_logins` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 4.1
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::must_change_option_is_set_to_on_for_all_sql_authenticated_logins {
  $instance = $title
  $sql_authenticated_logins_within_the_sysadmin_role = dig($facts, 'mssql_sql_authenticated_logins_within_the_sysadmin_role', $instance)

  mssql_secured_setup { "must_change_option_is_set_to_on_for_all_sql_authenticated_logins on ${title}":
    ensure => 'present',
  }
  alert('Specified cis is not applied, due to lack possiblity of audit.')
}
